/**
 * Macros for easy mapping of rstm mubench source to lpdstm source.
 * Mostly taken from rstm source distribution and slightly adapted.
 *
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#ifndef SWISSTM_MACROS_H_
#define SWISSTM_MACROS_H_

/////////////////////
// generate fields //
/////////////////////

#define TOKEN_CONCAT(a,b)  a ## b

#define GENERATE_FIELD(t, n)                                            \
  /* declare the field, with its name prefixed by m_ */                 \
  protected:                                                            \
    t TOKEN_CONCAT(m_, n);                                              \
  public:                                                               \
  /* declare a getter for a readable (const) version of the object */   \
  t TOKEN_CONCAT(get_, n)() const                                       \
  {                                                                     \
      return TOKEN_CONCAT(m_, n);                                       \
  }                                                                     \
  /* declare a getter for a writable version of the object */           \
  t TOKEN_CONCAT(get_, n)()                                             \
  {                                                                     \
      return TOKEN_CONCAT(m_, n);                                       \
  }                                                                     \
  /* declare a setter for a writable version of the object*/            \
  void TOKEN_CONCAT(set_, n)(t TOKEN_CONCAT(tmp_, n))                   \
  {                                                                     \
      TOKEN_CONCAT(m_, n) = TOKEN_CONCAT(tmp_, n);                      \
  }


#define GENERATE_ARRAY(t, n, s)                                         \
  /* declare the field, with its name prefixed by m_ */                 \
  protected:                                                            \
    t TOKEN_CONCAT(m_, n)[s];                                           \
  public:                                                               \
  /* declare a getter for a readale (const) version of the object */    \
  t TOKEN_CONCAT(get_, n)(int i) const                                  \
  {                                                                     \
      return TOKEN_CONCAT(m_, n)[i];                                  \
  }                                                                     \
  /* declare a getter for a writable version of the object */           \
  t TOKEN_CONCAT(get_, n)(int i)                                        \
  {                                                                     \
      return TOKEN_CONCAT(m_, n)[i];                                    \
  }                                                                     \
  /* declare a setter for a writable version of the object */           \
  void TOKEN_CONCAT(set_, n)(int i, t TOKEN_CONCAT(tmp_, n))            \
  {                                                                     \
      TOKEN_CONCAT(m_, n)[i] = TOKEN_CONCAT(tmp_, n);                   \
  }

#endif // SWISSTM_MACROS_H_
