/**
 * Timestamp class used for contention manager and validation
 * heuristics.
 * 
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#ifndef SWISSTM_TIMESTAMP_H_
#define SWISSTM_TIMESTAMP_H_

#include "../constants.h"
#include "atomic.h"

namespace swisstm {

	class GlobalTimestamp {
		public:
			GlobalTimestamp() : ts(MINIMUM_TS + 1) {  }

			unsigned readCurrentTs() const {
				return ts;
			}

			unsigned readCurrentTsAcquire() const {
				return (unsigned)atomic_load_acquire(&ts);
			}

			unsigned getNextTsFull() {
				return fetch_and_inc_full(&ts);
			}

			unsigned getNextTsAcquire() {
				return fetch_and_inc_acquire(&ts);
			}

			unsigned getNextTsNoBarrier() {
				return fetch_and_inc_no_barrier(&ts);
			}

			

		private:
			unsigned ts;
	} __attribute__ ((aligned(64)));
}

#endif // SWISSTM_TIMESTAMP_H_
