/**
 * Thread local storage implementation.
 * 
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#ifndef SWISSTM_TLS_H_
#define SWISSTM_TLS_H_

#include <pthread.h>

#ifdef USE_PTHREAD_TLS
namespace swisstm {

	template<class T>
	class Tls {
		public:
			static void GlobalInit();
			static void ThreadInit();
			static T *Get();

		private:
			static ::pthread_key_t tlsKey;
	};
}

template<class T> ::pthread_key_t swisstm::Tls<T>::tlsKey;

template<class T>
inline void swisstm::Tls<T>::GlobalInit() {
	::pthread_key_create(&tlsKey, NULL);
	T::GlobalInit();
}

template<class T>
inline void swisstm::Tls<T>::ThreadInit() {
	::pthread_setspecific(tlsKey, (const void *)(new T()));
}

template<class T>
inline T *swisstm::Tls<T>::Get() {
	return (T *)::pthread_getspecific(tlsKey);
}
#else

namespace swisstm {
	template<class T>
	class Tls {
		public:
			static void GlobalInit() {
				T::GlobalInit();
			}

			static void ThreadInit() {
				val = new T();
			}

			static T *Get() {
				return val;
			}

		private:
			static __thread T *val;
	};
}

template<class T> __thread T* swisstm::Tls<T>::val;

#endif

#endif // SWISSTM_TLS_H_
