/**
 * Memory management implementation.
 * 
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#include <cassert>

#include "memory_impl.h"
#include "../common/atomic.h"
#include "../constants.h"

// backing storage for timestamp
volatile swisstm::v_timestamp swisstm::global_timestamp
		__attribute__ ((aligned(64)));

void swisstm::global_init_mem() {
	Tls<thread_egc_data>::GlobalInit();
}

void swisstm::init_thread_mem() {
	// get thread identifier
	unsigned tid = swisstm::get_tid();

	// initialize tls data
	Tls<thread_egc_data>::ThreadInit();
	swisstm::thread_egc_data *egc = Tls<thread_egc_data>::Get();

	// initialize fields
	egc->freed = NULL;

	egc->free_on_abort = NULL;

	egc->free_on_commit = NULL;

	egc->free_on_commit_dominated = NULL;

	egc->pool_free_chunk = alloc_pool_free_chunk();
	egc->free_chunk_idx = 0;
	egc->pool_free_list = NULL;

	assert(tid < MAX_THREADS);

	// update global timestamp length
	unsigned current_ts_len;

	while(true) {
		current_ts_len = atomic_load_no_barrier(&global_timestamp.len);

		if(current_ts_len < tid + 1) {
			if(atomic_cas_no_barrier(&global_timestamp.len, current_ts_len, tid + 1)) {
				break;
			}
		} else {
			break;
		}
	}
}
