/**
 * Common implementation of transaction descriptors.
 * 
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#ifndef SWISSTM_TRANSACTION_H_
#define SWISSTM_TRANSACTION_H_

#include "memory.h"
#include "object.h"
#include "common/tls.h"
#include "aborted_exc.h"

#include "transaction_mixinv_ir_prio_t.h"

namespace swisstm {
	typedef TxMixinvIRPrioT Transaction;
}

/////////////////////////////////////
// thread local transaction object //
/////////////////////////////////////

namespace swisstm {

	// A shortcut used for accessing transaction in progress.
	class CurrentTransaction : public Tls<Transaction> {
		public:
			static void TxStart() {
				Tls<Transaction>::Get()->TxStart();
			}

			static void TxCommit() {
				Tls<Transaction>::Get()->TxCommit();
			}

			static void TxAbort() {
				Tls<Transaction>::Get()->TxAbort();
			}

			static void TxAbortFull() {
				Tls<Transaction>::Get()->TxAbortFull();
			}

			static BaseObject *OpenWrite(ObjectHandleBase *handle) {
				return Tls<Transaction>::Get()->OpenWrite(handle);
			}

			static void Delete(ObjectHandleBase *handle) {
				Tls<Transaction>::Get()->Delete(handle);
			}

			static const BaseObject *OpenRead(ObjectHandleBase *handle) {
				return Tls<Transaction>::Get()->OpenRead(handle);
			}
	};
}

#endif // SWISSTM_TRANSACTION_H_
